/*
 *	Implementation of plugin interface
 *	Copyright (C) 2011 CEDAR Solutions s.r.l.
 *
 *	Authors:
 *		Marco Giammarini <m.giammarini@warcomeb.it>
 */

#include "stdafx.h"
#include "resource.h"
#include "TrajectoryControl.h"
#include "Mach4View.h"
#include "Engine.h"
#include "rs274ngc.h"
#include "XMLProfile.h"

#include "MachDevImplementation.h"

#include ".\PluginConsole.h"
#include ".\USBPlugin.h"

/** Native USB device driver for FTDI FT232x */
#include ".\ftd2xx.h"

#include <mmsystem.h>
#include <math.h>
#include <vector>

using std::vector;
   
extern CMach4View *MachView;

/** Plugin utility define */
#define PLUGIN_DEBUG

/** Plugin object */
USBPlugin * plugin = NULL;


/* Device information inside FTDI EEprom: */
/* WARNING: Manufacturer string length plus the Description
   string length must be less than or equal to 40 characters. */
//#define CEDAR_MANUFACTURER      "Cedar s.r.l."
//#define CEDAR_MANUFACTURER_ID   "CEDAR"
//#define CEDAR_DESCRIPTION       "Mach3 USBStepMotorCtrl"
//#define CEDAR_SERIAL_NUMBER     "CSM30001"
#define CEDAR_BBM_TYPE          0x1
#define CEDAR_BBM_DIRECTION     0xFF
#define CEDAR_BAUDRATE          FT_BAUD_115200
/* Define to work with FT232H Mini Module: */
#define CEDAR_MANUFACTURER      "FTDI"
#define CEDAR_MANUFACTURER_ID   "FT"
#define CEDAR_DESCRIPTION       "FT2232H MiniModule A"
#define CEDAR_SERIAL_NUMBER     "FTT9ZLN7A"

/* Config dialog box controls define. */
#define CONFIG_BOX_TEXT_ID      200

/******* Device variables: *******/
/** Handle to open USBStepMotorCtrl Device */
static FT_HANDLE                ftHandle;
/** FTDI response */
static FT_STATUS                ftStatus;
/** Numbers of D2XX device connected */
static DWORD                    numDevs;
/** Devices informations */
static FT_DEVICE_LIST_INFO_NODE *devInfos;
/** List of usable device (position  in devInfos). */
static vector< DWORD >          devUsable;
/** Number of byte that we have writen in a device queue. */
static DWORD                    dwBytesInQueue;


/******* DEBUG *******/
#define BUFFER_SIZE	512
unsigned char data [BUFFER_SIZE];


/* Internal utility functions: */
static void errorMessage (void);

/******* Implementation of utility internal functions *******/

/**
 *
 */
static void errorMessage (void)
{
	MessageBox(
		NULL,
		"PlugIn does not work: no device connected, or device does not work!\n"
		"Please, control your device and tray again!",
		"USBStepMotorCtrl DOES NOT WORK",
		MB_ICONERROR | MB_OK
	);
}

/******* Here are the routines which are called by Mach3 via MachDevice. *******/

/**
 * Used for destrcution of variables prior to exit. Called as Mach3 shuts down. 
 *
 */
void myCleanUp()
{
   /* If we have opened a connection with a device, we close it! */
   if(ftHandle != NULL)
	   FT_Close(ftHandle);
}


CString	myProfileInit (CString name, CXMLProfile *DevProf)	
// initial access to Mach profile when enumerating available plugins
// Returns the second half on the "pluging id" for the list in Operator
// MachDevice adds the file name at the start of the string

{
	//this gets the default directory DefDir in which Mach3 is located.  and the profile name ex. "Mach3Mill"
	//DefDir = DevProf->GetProfileString("Preferences","DefDir","C:\\Mach3\\");
    //Profile = DevProf->GetProfileString("Preferences","Profile","Mach3Mill");
	return " USBMotorControl by Cedar Solutions s.r.l. 1.0.0";
} // myProfileInit

void	myInitControl ()
{
	/* Disable printer port... it is the default behavior! */
	MachView->m_PrinterOn = false;
	/* For 1ms timed waypoints... */
	MainPlanner->ExTime = .001;
	/* This seems to be movement buffer length (in movements) */
	MainPlanner->ExBufferHi = 350;
	/* For variable ms timing... */
	MainPlanner->ExternalType = EX_VMS;
	/* Setup external pulse rate */
	for( int x = 0; x < 7; x++ )
	{
		MainPlanner->ExternalPulseRates[x] = 1000000;
	}

	return;
}

void	myPostInitControl ()
{
	if (plugin == NULL)
	{
		plugin = new USBPlugin();

		if (!plugin->connect())
		{
			delete plugin;
			plugin = NULL;
#ifdef PLUGIN_DEBUG
			PluginConsole::getInstance()->printMessage("Plugin: Device not connected!");
#endif
		}
		else
		{
#ifdef PLUGIN_DEBUG
			PluginConsole::getInstance()->printMessage("Plugin: Activated and device connected!");
#endif
		}
	}
}

/**
 *
 *
 */
void myConfig (CXMLProfile *DevProf)
{
	/* If we have already opened communication with device, we ask to the user if we close it. */
	if(ftHandle != NULL)
	{
		int messageResponse = MessageBox(
			NULL,
			"The PlugIn is aready configured,would you like reconfigure it?",
			"USBStepMotorCtrl, connection already exist!",
			MB_ICONWARNING | MB_YESNO | MB_DEFBUTTON2
		);

		if (messageResponse == IDNO)
			return;
		else
			FT_Close(ftHandle);
	}

	/* Clear usable vector before start config dialog box. */
	if(devUsable.size()>0)
		devUsable.clear();

	ftStatus = FT_CreateDeviceInfoList(&numDevs);
	if(ftStatus != FT_OK)
	{
		errorMessage();
		return;
	}

	/* Allocate memory for device informations */
	if(numDevs > 0)
	{
		devInfos = (FT_DEVICE_LIST_INFO_NODE*)malloc(sizeof(FT_DEVICE_LIST_INFO_NODE)*numDevs);
		/* Get devices informations */
		ftStatus = FT_GetDeviceInfoList(devInfos,&numDevs);
		if(ftStatus != FT_OK)
		{
			errorMessage();
			return;
		}
	}
	else
	{
		errorMessage();
		return;
	}

	/* Search correct device... */
	for(DWORD i = 0; i < numDevs; i++)
	{
		if(strcmp(devInfos[i].Description,CEDAR_DESCRIPTION) == 0)
		{
			devUsable.push_back(i);
		}
	}

	/* If we have usable device, call config box... */
	if(devUsable.size()>0)
	{
		/* We don't use this function... */
		//configBoxMessage(NULL,NULL);
		/* We open the first device of the list! */
		ftStatus = FT_OpenEx(devInfos[devUsable[0]].Description,FT_OPEN_BY_DESCRIPTION,&ftHandle);
		if(ftStatus == FT_OK)
		{
			/* prepare info message... */
			char tmpMessage [400];
			sprintf(
				tmpMessage,
				"The PlugIn woking properly with the device:\n\nDescription = %s\nSerialNumber = %s\nID = 0x%x\nLocId = 0x%x\n",
				devInfos[devUsable[0]].Description,
				devInfos[devUsable[0]].SerialNumber,
				devInfos[devUsable[0]].ID,
				devInfos[devUsable[0]].LocId
			);

			MessageBox(
				NULL,
				tmpMessage,
				"USBStepMotorCtrl configured!",
				MB_ICONINFORMATION | MB_OK
			);
		}
		else
		{
			errorMessage();
			return;
		}
	}

	/* Set Bit Bang Mode type... */ 
	ftStatus = FT_SetBitMode(ftHandle, CEDAR_BBM_DIRECTION, CEDAR_BBM_TYPE);
	if (ftStatus != FT_OK) 
	{
		errorMessage();
		return;
	}

	/* Set BaudRate. Data rate on parallel pins is (Baudrate * 16) */
	FT_SetBaudRate(ftHandle, CEDAR_BAUDRATE);

	/* Purge TX and RX buffers */
	ftStatus = FT_Purge(ftHandle, FT_PURGE_TX | FT_PURGE_RX);  
	if (ftStatus != FT_OK) 
	{
		errorMessage();
		return;
	}
}

/**
 *
 *
 */
void	myUpdate ()
{
	if (plugin != NULL)
	{
		plugin->update();
	}
}

void	myHighSpeedUpdate ()
// called at 40Hz. This is used for Higher Res timing needs. 
// only use as required, it can use alot of system reosurces. 
//not really required in the G100, so Ill leave it blank for now..

{

} // myHighSpeed Update

/*void   myNotify (int id)
{
	MessageBox(NULL,"It's ALIVE","Menu OK",MB_OK);
}*/



// ======================================================================================================
//
// The Helpers follow
//
//=======================================================================================================

// put your helper routines here

//This is a helper for Debug Messages. Mach3's MainPlanner->DebugRun var makes messages fill the file up during a run.




//All the following code runs th evarious phases of the G100. 

///PHASE Code. The thing to remember about phase fucntions below, is they are all called 10 times a second whan their phase is active;

//Discovery. This guys job is to open the G100 in the event of boot, attach, or reflash timeouts. It allows for an asyncronous way 
//           to attach to a G100 thats on the network. Furture work: Attach to selection of devices. 
